package ru.dbotthepony.mc.otm.screen;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.ChatFormatting;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.GameRenderer;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.player.Inventory;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.Registry;
import ru.dbotthepony.mc.otm.capability.MatteryCapability;
import ru.dbotthepony.mc.otm.capability.android.AndroidCapabilityPlayer;
import ru.dbotthepony.mc.otm.capability.android.AndroidFeatureResearchNode;
import ru.dbotthepony.mc.otm.menu.AndroidStationMenu;
import ru.dbotthepony.mc.otm.screen.panels.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

public class AndroidStationScreen extends MatteryScreen<AndroidStationMenu> {
	class AndroidFeatureButton extends EditablePanel {
		public static final int BUTTON_SIZE = 16;
		private final AndroidFeatureResearchNode node;

		public static final RGBAColor RESEARCHED = new RGBAColor(150, 150, 200);
		public static final RGBAColor CAN_BE_RESEARCHED = new RGBAColor(150, 200, 150);
		public static final RGBAColor CAN_NOT_BE_RESEARCHED = new RGBAColor(200, 150, 150);

		public AndroidFeatureButton(@Nullable EditablePanel parent, AndroidFeatureResearchNode node) {
			super(AndroidStationScreen.this, parent, 0, 0, BUTTON_SIZE, BUTTON_SIZE);
			this.node = node;
			setDockMargin(2, 2, 0, 0);
		}

		@Override
		protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
			Minecraft.getInstance().player.getCapability(MatteryCapability.ANDROID).ifPresent(_cap -> {
				if (_cap instanceof AndroidCapabilityPlayer cap) {
					if (cap.hasFeature(node.type)) {
						RenderHelper.setDrawColor(RESEARCHED);
					} else if (node.canResearch(Minecraft.getInstance().player, cap)) {
						RenderHelper.setDrawColor(CAN_BE_RESEARCHED);
					} else {
						RenderHelper.setDrawColor(CAN_NOT_BE_RESEARCHED);
					}

					RenderHelper.drawRect(stack, 0, 0, getWidth(), getHeight());
				}
			});
		}

		@Override
		protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
			node.researchClient();
			return true;
		}

		@Override
		protected boolean innerRenderTooltips(PoseStack stack, float mouse_x, float mouse_y, float flag) {
			if (is_hovered) {
				var list = new ArrayList<>(node.getTooltip());

				Minecraft.getInstance().player.getCapability(MatteryCapability.ANDROID).ifPresent(_cap -> {
					if (_cap instanceof AndroidCapabilityPlayer cap) {
						if (cap.hasFeature(node.type)) {
							list.add(new TranslatableComponent("otm.android_station.research.researched").withStyle(ChatFormatting.DARK_AQUA));
						} else if (node.canResearch(Minecraft.getInstance().player, cap)) {
							list.add(new TranslatableComponent("otm.android_station.research.can_be_researched").withStyle(ChatFormatting.DARK_GREEN));
						} else {
							list.add(new TranslatableComponent("otm.android_station.research.can_not_be_researched").withStyle(ChatFormatting.DARK_RED));
						}
					}
				});

				renderComponentTooltip(stack, list, (int) mouse_x, (int) mouse_y);
			}

			return is_hovered;
		}
	}

	private final Set<AndroidFeatureResearchNode> nodes = new HashSet<>();

	public static final int FRAME_WIDTH = 210;
	public static final int GRID_WIDTH = 6;
	public static final int GRID_HEIGHT = 5;
	public static final int FRAME_HEIGHT = 120;

	public AndroidStationScreen(AndroidStationMenu p_97741_, Inventory p_97742_, Component p_97743_) {
		super(p_97741_, p_97742_, p_97743_);

		for (var feature : Registry.ANDROID_FEATURES().getValues()) {
			var node = feature.getResearchNode();

			if (node != null) {
				nodes.add(node);
			}
		}
	}

	@Nullable
	@Override
	protected FramePanel makeMainFrame() {
		var frame = new FramePanel(this, null, 0, 0, FRAME_WIDTH, FRAME_HEIGHT, getTitle());

		var grid = new GridPanel(this, frame, 0, 0, GRID_WIDTH * 18, 0, GRID_WIDTH, GRID_HEIGHT);

		for (var feature : Registry.ANDROID_FEATURES().getValues()) {
			var node = feature.getResearchNode();

			if (node != null) {
				new AndroidFeatureButton(grid, node);
			}
		}

		grid.setDocking(Dock.RIGHT);

		return frame;
	}
}
